require 'bolster/version'
require 'net/http'
require 'uri'

module Bolster
    class Server
        def initialize(uri, options = {})
            @uri = URI(uri)
            @options = options
        end

        def delete(uri)
            res = request(Net::HTTP::Delete.new(full_uri(uri)))
            res.body
        end
        
        def get(uri)
            res = request(Net::HTTP::Get.new(full_uri(uri)))
            res.body
        end
    
        def put(uri, json)
            req = Net::HTTP::Put.new(full_uri(uri))
            req['content-type'] = 'application/json'
            req.body = json
            request(req)
        end

        def post(uri, json)
            req = Net::HTTP::Post.new(full_uri(uri))
            req['content-type'] = 'application/json'
            req.body = json
            request(req)
        end

        def full_uri(uri)
            full_uri = @uri.request_uri + uri
            full_uri.gsub!('//','/')
        end
        
        def request(req)
            http = Net::HTTP.new(@uri.hostname, @uri.port)
            if @uri.class == URI::HTTPS
                require 'net/https'
                http.use_ssl = true
            end
            
            if @options.has_key?(:user) && @options.has_key?(:password)
                req.basic_auth(@options[:user], @options[:password])
            end
            
            res = http.start { |h| h.request(req) }
            unless res.kind_of?(Net::HTTPSuccess)
                handle_error(req, res)
            end
            res
        end

        def handle_error(req, res)
            e = RuntimeError.new(%Q{#{res.code}: #{res.message}
                METHOD: #{req.method}
                URI: #{req.path}
                #{req.body}})
            raise e
        end
    end
end
